/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil -*- */

/*
 * High-level SWIG Interface to NDS client library
 *
 * Copyright (C) 2015  Edward Maros <ed.maros@ligo.org>
 *
 * This work is derived from the C NDS clientlibrary created by Leo Singer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#ifndef SWIG__COMMON__NDS_CONNECTION_I
#define SWIG__COMMON__NDS_CONNECTION_I

%include "std_string.i"

%{

#include "nds_connection.hh"
#include "nds_channel.hh"

using namespace NDS;
%}

%template(buffers_vector) std::vector< buffer >;
%template(buffers_rovector) rovector< buffer >;
//%template(connection_parameters_type) std::vector< std::string >;

class buffers_type
  : public rovector< buffer >
{
public:
};

class channels_type
  : public rovector< channel >
{
public:
};

class connection
{
public:
  typedef enum {
    PROTOCOL_INVALID = -1,
    PROTOCOL_ONE = 1,
    PROTOCOL_TWO = 2,
    PROTOCOL_TRY = 3
  } protocol_type;

  typedef std::string host_type;
  typedef int port_type;
  typedef size_t count_type;
  typedef channel::channel_names_type channel_names_type;
  typedef std::vector<std::string> parameters_type;

  static const port_type DEFAULT_PORT = 31200;

  connection( const host_type& host,
              port_type port = DEFAULT_PORT,
              protocol_type protocol = PROTOCOL_TRY );
  ~connection( );

  void close( );

  count_type
  count_channels( std::string channel_glob = "*",
                 channel::channel_type channel_type_mask = channel::DEFAULT_CHANNEL_MASK,
                 channel::data_type data_type_mask = channel::DEFAULT_DATA_MASK,
                 channel::sample_rate_type min_sample_rate = channel::MIN_SAMPLE_RATE,
                 channel::sample_rate_type max_sample_rate = channel::MAX_SAMPLE_RATE );

  channels_type
  find_channels( std::string channel_glob = "*",
                 channel::channel_type channel_type_mask = channel::DEFAULT_CHANNEL_MASK,
                 channel::data_type data_type_mask = channel::DEFAULT_DATA_MASK,
                 channel::sample_rate_type min_sample_rate = channel::MIN_SAMPLE_RATE,
                 channel::sample_rate_type max_sample_rate = channel::MAX_SAMPLE_RATE );

  epochs_type
  get_epochs();

  bool
  set_epoch(std::string epoch);

  bool
  set_epoch(buffer::gps_second_type gps_start,
        buffer::gps_second_type gps_stop);

  epoch
  current_epoch() const;

  bool
  check( buffer::gps_second_type gps_start,
         buffer::gps_second_type gps_stop,
         const channel_names_type& channel_names );

  buffers_type
  fetch( buffer::gps_second_type gps_start,
         buffer::gps_second_type gps_stop,
         const channel_names_type& channel_names );

  host_type get_host( ) const;

  port_type get_port( ) const;

  protocol_type get_protocol( ) const;

  bool set_parameter(const std::string &parameter, const std::string &value);

  std::string get_parameter( const std::string &parameter ) const;

  connection::parameters_type get_parameters( ) const;

  const channel::hash_type& hash( ) const;

  connection&
  iterate( const channel_names_type& channel_names );

  connection&
  iterate( buffer::gps_second_type stride,
           const channel_names_type& channel_names );

  connection&
  iterate( buffer::gps_second_type gps_start,
           buffer::gps_second_type gps_stop,
           const channel_names_type& channel_names );

  connection&
  iterate( buffer::gps_second_type gps_start,
           buffer::gps_second_type gps_stop,
           buffer::gps_second_type stride,
           const channel_names_type& channel_names );

  bool
  has_next( );

  buffers_type
  next( );

  bool request_in_progress( ) const;

  void shutdown( );

  availability_list_type
  get_availability(const channel_names_type& channel_names );

};

#endif /* SWIG__COMMON__NDS_CONNECTION_I */
