/* -*- mode: C++ ; c-basic-offset: 2; indent-tabs-mode: nil -*- */
/*
 * This file allows for SWIG specialization for Python
 */
#if defined(SWIGPYTHON)

%feature("autodoc","1");

%include "nds.i"
 // %include "numpy.i"

%begin %{
extern "C"
{
#define NPY_NO_DEPRECATED_API NPY_API_VERSION

#include <Python.h>
#include <numpy/arrayobject.h>

#if NPY_API_VERSION >= 0x00000007
// Do nothing since the macro is already available
#else // NPY_API_VERSION >= 0x00000007
#define PyArray_SetBaseObject(A,B)	PyArray_BASE(A) = B
#endif // NPY_API_VERSION >= 0x00000007

}
%}

%include "nds_python_buffer.i"

%init %{
  import_array( );
%}

%extend channel {
  const char* __str__( )
  {
    return $self->NameLong( ).c_str( );
  }

  %pythoncode %{
     try:
       __swig_getmethods__
     except NameError:
       __swig_getmethods__ = {}

     __swig_getmethods__['name'] = Name
     __swig_getmethods__['channel_type'] = Type
     __swig_getmethods__['data_type'] = DataType
     __swig_getmethods__['sample_rate'] = SampleRate
     #__swig_getmethods__['gain'] = Gain
     #__swig_getmethods__['slope'] = Slope
     #__swig_getmethods__['offset'] = Offset
     #__swig_getmethods__['units'] = Units
     __swig_getmethods__['signal_gain'] = Gain
     __swig_getmethods__['signal_slope'] = Slope
     __swig_getmethods__['signal_offset'] = Offset
     __swig_getmethods__['signal_units'] = Units

     try:
       object
       name = property(Name)
       channel_type = property(Type)
       data_type = property(DataType)
       sample_rate = property(SampleRate)
       #gain = property(Gain)
       #slope = property(Slope)
       #offset = property(Offset)
       #units = property(Units)
       signal_gain = property(Gain)
       signal_slope = property(Slope)
       signal_offset = property(Offset)
       signal_units = property(Units)
     except AttributeError:
       pass

  %}
}

%extend connection
{
  %insert("python") %{
    def __iter__(self):
      return self
  %}
}

typedef const unsigned char* lcl_data_vector_type;

%typemap(out) lcl_data_vector_type {
  npy_intp dims[] = {arg1->length};
  int typenum;

  switch (arg1->channel.data_type)
  {
  case NDS::channel::DATA_TYPE_INT16:
    typenum = NPY_INT16;
    break;
  case NDS::channel::DATA_TYPE_INT32:
    typenum = NPY_INT32;
    break;
  case NDS::channel::DATA_TYPE_INT64:
    typenum = NPY_INT64;
    break;
  case NDS::channel::DATA_TYPE_FLOAT32:
    typenum = NPY_FLOAT32;
    break;
  case NDS::channel::DATA_TYPE_FLOAT64:
    typenum = NPY_FLOAT64;
    break;
  case NDS::channel::DATA_TYPE_COMPLEX32:
    typenum = NPY_COMPLEX64;
    break;
  case NDS::channel::DATA_TYPE_UINT32:
    typenum = NPY_UINT32;
    break;
  default:
    SWIG_exception(SWIG_TypeError, "Unknown NDS data type");
    break;
  }
  
  $result = PyArray_SimpleNewFromData(1, dims, typenum, $1);
  if (!$result)
  {
    SWIG_fail;
  }

  Py_INCREF($self);
  PyArray_SetBaseObject($result, $self);
}

#endif /* defined(SWIGPYTHON) */

